@@ -5,6 +5,8 @@ import numpy as np
 from surfa import transform
 from surfa.core.array import check_array
 from surfa.transform.space import cast_space
+from surfa.core.istype import ismesh
+from surfa.core.istype import isimage
 
 
 class Affine:
@@ -38,7 +40,8 @@ class Affine:
         self.target = target
 
     def __repr__(self):
-        return repr(self.matrix).replace('array([', 'sf.Affine([\n       ')
+        with np.printoptions(precision=4, suppress=True):
+            return repr(self.matrix).replace('array([', 'sf.Affine([\n       ')
 
     @property
     def _writeable(self):
@@ -73,6 +76,10 @@ class Affine:
         if self.matrix is not None:
             self._check_writeability()
 
+        # check if input is actually an Affine object
+        if isinstance(mat, Affine):
+            mat = mat.matrix.copy()
+
         # check input shape
         mat = np.ascontiguousarray(mat, dtype='float')
         check_array(mat, ndim=2, name='affine matrix')
@@ -92,6 +99,19 @@ class Affine:
         """
         return self.matrix
 
+    def __getitem__(self, index):
+        """
+        Use indexing similar to numpy arrays.
+        """
+        return self.matrix[index]
+
+    def __setitem__(self, index, value):
+        """
+        Use indexing similar to numpy arrays.
+        """
+        self._check_writeability()
+        self.matrix[index] = value
+
     @property
     def space(self):
         """
@@ -148,6 +168,15 @@ class Affine:
         aff._writeable = True
         return aff
 
+    def new(self, matrix):
+        """
+        Return a new instance of the affine with an updated matrix and
+        preserved metadata.
+        """
+        aff = self.copy()
+        aff.matrix = matrix
+        return aff
+
     @property
     def ndim(self):
         """
@@ -186,11 +215,28 @@ class Affine:
         (..., N) float
             Transformed N-D point array.
         """
-        points = np.asarray(points)
-        if points.ndim == 1:
-            points = points[np.newaxis]
+        # a common mistake is to use this function for transforming an image
+        # or mesh, so run this check to help the user out a bit
+        if ismesh(points):
+            raise ValueError('use mesh.transform(affine) to apply an affine to a mesh')
+        if isimage(points):
+            raise ValueError('use image.transform(affine) to apply an affine to an image')
+
+        # convert to array
+        points = np.ascontiguousarray(points)
+
+        # check correct dimensionality
+        if points.shape[-1] != self.ndim:
+            raise ValueError(f'transform() method expected {self.ndim}D points, but got '
+                             f'{points.shape[-1]}D input with shape {points.shape}')
+
+        # account for multiple possible input axes and be sure to
+        # always return the same shape
+        shape = points.shape
+        points = points.reshape(-1, self.ndim)
         points = np.c_[points, np.ones(points.shape[0])].T
-        return np.ascontiguousarray(np.dot(self.matrix, points).T.squeeze()[..., :-1])
+        moved = np.dot(self.matrix, points).T[:, :-1]
+        return np.ascontiguousarray(moved).reshape(shape)
 
     def inv(self):
         """
@@ -231,17 +277,18 @@ class Affine:
             Tuple of (translation, rotation, scale, shear) transform components.
         """
         translation = self.matrix[: self.ndim, -1]
-
         q, r = np.linalg.qr(self.matrix[: self.ndim, : self.ndim])
 
+        # scaling
         di = np.diag_indices(self.ndim)
         scale = np.abs(r[di])
-
         p = np.eye(self.ndim)
         p[di] = r[di] / scale
 
+        # rotation
         rotation = rotation_matrix_to_angles(q @ p, degrees=degrees)
 
+        # shear
         shear_mat = (p @ r) / np.expand_dims(scale, -1)
         if self.ndim == 2:
             shear = shear_mat[0, 1]
@@ -649,3 +696,26 @@ def random_affine(
         **kwargs,
     )
     return aff
+
+
+def center_to_corner_rotation(affine, image_shape):
+    """
+    Convert an affine matrix that rotates around an image center to
+    a standard-format affine that rotates around the corner (origin).
+
+    Parameters
+    ----------
+    affine : Affine
+        Affine matrix with center rotation.
+    input_shape: tuple of int
+        Shape of the base image.
+
+    Returns
+    -------
+    Affine
+        Converted affine matrix with corner rotation.
+    """
+    center = identity()
+    center[:3, -1] = -0.5 * (np.asarray(image_shape[:affine.ndim]) - 1)
+    shifted = center.inv() @ (affine @ center)
+    return affine.new(shifted)
